@@ -1,4 +1,4 @@
-#   Copyright 2020 The PyMC Developers
+#   Copyright 2023 The PyMC Developers
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -18,15 +18,17 @@ Created on Mar 7, 2011
 @author: johnsalvatier
 """
 
+import warnings
+
 from abc import ABC, abstractmethod
 from enum import IntEnum, unique
-from typing import Dict, List, Sequence, Tuple, Union
+from typing import Any, Dict, Iterable, List, Mapping, Sequence, Tuple, Union
 
 import numpy as np
 
 from pytensor.graph.basic import Variable
 
-from pymc.blocking import PointType, StatsDict, StatsType
+from pymc.blocking import PointType, StatDtype, StatsDict, StatShape, StatsType
 from pymc.model import modelcontext
 
 __all__ = ("Competence", "CompoundStep")
@@ -48,10 +50,61 @@ class Competence(IntEnum):
     IDEAL = 3
 
 
-class BlockedStep(ABC):
+def infer_warn_stats_info(
+    stats_dtypes: List[Dict[str, StatDtype]],
+    sds: Dict[str, Tuple[StatDtype, StatShape]],
+    stepname: str,
+) -> Tuple[List[Dict[str, StatDtype]], Dict[str, Tuple[StatDtype, StatShape]]]:
+    """Helper function to get `stats_dtypes` and `stats_dtypes_shapes` from either of them."""
+    # Avoid side-effects on the original lists/dicts
+    stats_dtypes = [d.copy() for d in stats_dtypes]
+    sds = sds.copy()
+    # Disallow specification of both attributes
+    if stats_dtypes and sds:
+        raise TypeError(
+            "Only one of `stats_dtypes_shapes` or `stats_dtypes` must be specified."
+            f" `{stepname}.stats_dtypes` should be removed."
+        )
 
+    # Infer one from the other
+    if not sds and stats_dtypes:
+        warnings.warn(
+            f"`{stepname}.stats_dtypes` is deprecated."
+            " Please update it to specify `stats_dtypes_shapes` instead.",
+            DeprecationWarning,
+        )
+        if len(stats_dtypes) > 1:
+            raise TypeError(
+                f"`{stepname}.stats_dtypes` must be a list containing at most one dict."
+            )
+        for sd in stats_dtypes:
+            for sname, dtype in sd.items():
+                sds[sname] = (dtype, None)
+    elif sds:
+        stats_dtypes.append({sname: dtype for sname, (dtype, _) in sds.items()})
+    return stats_dtypes, sds
+
+
+class BlockedStep(ABC):
     stats_dtypes: List[Dict[str, type]] = []
+    """A list containing <=1 dictionary that maps stat names to dtypes.
+
+    This attribute is deprecated.
+    Use `stats_dtypes_shapes` instead.
+    """
+
+    stats_dtypes_shapes: Dict[str, Tuple[StatDtype, StatShape]] = {}
+    """Maps stat names to dtypes and shapes.
+
+    Shapes are interpreted in the following ways:
+    - `[]` is a scalar.
+    - `[3,]` is a length-3 vector.
+    - `[4, None]` is a matrix with 4 rows and a dynamic number of columns.
+    - `None` is a sparse stat (i.e. not always present) or a NumPy array with varying `ndim`.
+    """
+
     vars: List[Variable] = []
+    """Variables that the step method is assigned to."""
 
     def __new__(cls, *args, **kwargs):
         blocked = kwargs.get("blocked")
@@ -78,12 +131,21 @@ class BlockedStep(ABC):
         if len(vars) == 0:
             raise ValueError("No free random variables to sample.")
 
+        # Auto-fill stats metadata attributes from whichever was given.
+        stats_dtypes, stats_dtypes_shapes = infer_warn_stats_info(
+            cls.stats_dtypes,
+            cls.stats_dtypes_shapes,
+            cls.__name__,
+        )
+
         if not blocked and len(vars) > 1:
             # In this case we create a separate sampler for each var
             # and append them to a CompoundStep
             steps = []
             for var in vars:
                 step = super().__new__(cls)
+                step.stats_dtypes = stats_dtypes
+                step.stats_dtypes_shapes = stats_dtypes_shapes
                 # If we don't return the instance we have to manually
                 # call __init__
                 step.__init__([var], *args, **kwargs)
@@ -94,6 +156,8 @@ class BlockedStep(ABC):
             return CompoundStep(steps)
         else:
             step = super().__new__(cls)
+            step.stats_dtypes = stats_dtypes
+            step.stats_dtypes_shapes = stats_dtypes_shapes
             # Hack for creating the class correctly when unpickling.
             step.__newargs = (vars,) + args, kwargs
             return step
@@ -127,6 +191,25 @@ class BlockedStep(ABC):
             self.tune = False
 
 
+def flat_statname(sampler_idx: int, sname: str) -> str:
+    """Get the flat-stats name for a samplers stat."""
+    return f"sampler_{sampler_idx}__{sname}"
+
+
+def get_stats_dtypes_shapes_from_steps(
+    steps: Iterable[BlockedStep],
+) -> Dict[str, Tuple[StatDtype, StatShape]]:
+    """Combines stats dtype shape dictionaries from multiple step methods.
+
+    In the resulting stats dict, each sampler stat is prefixed by `sampler_#__`.
+    """
+    result = {}
+    for s, step in enumerate(steps):
+        for sname, (dtype, shape) in step.stats_dtypes_shapes.items():
+            result[flat_statname(s, sname)] = (dtype, shape)
+    return result
+
+
 class CompoundStep:
     """Step method composed of a list of several other step
     methods applied in sequence."""
@@ -136,6 +219,7 @@ class CompoundStep:
         self.stats_dtypes = []
         for method in self.methods:
             self.stats_dtypes.extend(method.stats_dtypes)
+        self.stats_dtypes_shapes = get_stats_dtypes_shapes_from_steps(methods)
         self.name = (
             f"Compound[{', '.join(getattr(m, 'name', 'UNNAMED_STEP') for m in self.methods)}]"
         )
@@ -181,26 +265,49 @@ def flatten_steps(step: Union[BlockedStep, CompoundStep]) -> List[BlockedStep]:
 class StatsBijection:
     """Map between a `list` of stats to `dict` of stats."""
 
-    def __init__(self, sampler_stats_dtypes: Sequence[Dict[str, type]]) -> None:
+    def __init__(self, sampler_stats_dtypes: Sequence[Mapping[str, type]]) -> None:
         # Keep a list of flat vs. original stat names
-        self._stat_groups: List[List[Tuple[str, str]]] = [
-            [(f"sampler_{s}__{statname}", statname) for statname, _ in names_dtypes.items()]
-            for s, names_dtypes in enumerate(sampler_stats_dtypes)
-        ]
+        stat_groups = []
+        for s, names_dtypes in enumerate(sampler_stats_dtypes):
+            group = []
+            for statname, dtype in names_dtypes.items():
+                flatname = flat_statname(s, statname)
+                is_obj = np.dtype(dtype) == np.dtype(object)
+                group.append((flatname, statname, is_obj))
+            stat_groups.append(group)
+        self._stat_groups: List[List[Tuple[str, str, bool]]] = stat_groups
+        self.object_stats = {
+            fname: (s, sname)
+            for s, group in enumerate(self._stat_groups)
+            for fname, sname, is_obj in group
+            if is_obj
+        }
 
-    def map(self, stats_list: StatsType) -> StatsDict:
+    @property
+    def n_samplers(self) -> int:
+        return len(self._stat_groups)
+
+    def map(self, stats_list: Sequence[Mapping[str, Any]]) -> StatsDict:
         """Combine stats dicts of multiple samplers into one dict."""
         stats_dict = {}
         for s, sts in enumerate(stats_list):
-            for statname, sval in sts.items():
-                sname = f"sampler_{s}__{statname}"
-                stats_dict[sname] = sval
+            for fname, sname, is_obj in self._stat_groups[s]:
+                if sname not in sts:
+                    continue
+                stats_dict[fname] = sts[sname]
         return stats_dict
 
-    def rmap(self, stats_dict: StatsDict) -> StatsType:
-        """Split a global stats dict into a list of sampler-wise stats dicts."""
+    def rmap(self, stats_dict: Mapping[str, Any]) -> StatsType:
+        """Split a global stats dict into a list of sampler-wise stats dicts.
+
+        The ``stats_dict`` can be a subset of all sampler stats.
+        """
         stats_list = []
-        for namemap in self._stat_groups:
-            d = {statname: stats_dict[sname] for sname, statname in namemap}
+        for group in self._stat_groups:
+            d = {}
+            for fname, sname, is_obj in group:
+                if fname not in stats_dict:
+                    continue
+                d[sname] = stats_dict[fname]
             stats_list.append(d)
         return stats_list
